# coding: latin1
# ##############################################################
# Achtung:
# PythonForS60 interpretiert Quelltextdateien nur in latin1!!
# ##############################################################

# Dieses Dokument wird verffentlicht unter der CC-Lizenz by-nc-sa
# http://creativecommons.org/licenses/by-nc-sa/3.0/de/

# Fr PythonForS60 Version 1.4.5 muss cElementTree nachinstalliert
# werden. Ein Installationspaket mit abgelaufenem Zertifikat,
# (also noch neu zu signieren -- "ensymble signsis ...")
# ist verfgbar unter:
# http://ssalmine.googlepages.com/somepys60extensions.

# Importiere XML-Parser Funktionalitaet,
try:
    import cElementTree as ET
except ImportError:
    import xml.etree.cElementTree as ET
            
# GUI Methoden, bei einer Portierung entsprechend zu ersetzen
import appuifw
def stelleFrageGUI(text):
    """
    Diese Funktion stellt der Nutzerin bzw. dem Nutzer
    den bergebenen String als Frage mit Mglichkeit zur
    Ja/Nein-Antwort. Die Antwort "Ja" wird als True,
    die Antwort "Nein" als False zurckgegeben.
    
    Diese Funktion nutzt die appuifw.query-Methode mit
    type="query" zur Interaktion, schn wre noch eine
    Mglichkeit, die Softkey-Label anzupassen, aber nicht heute...
    """
    r = appuifw.query(unicode(text)+"\n(Ok=Ja, Abbruch=Nein)","query")
    if r==1:
        return True
    if r is None:
        return False

def liesTextGUI(label):
    """
    Diese Funktion liest von dem Benutzer bzw. von der Benutzerin
    einen Text ein. Die Aufforderung wird mit dem bergebenem String
    betitelt.
    
    Diese Funktion nutzt die appuifw.query-Methode mit type="text".
    """
    return appuifw.query(unicode(label),"text")

def gibAusGUI(text):
    """
    Diese Funktion gibt den bergebenen Text als
    Hinweis fr Nutzerin oder Nutzer aus.
    
    Zur Ausgabe wird die appuifw.note-Methode benutzt.
    """
    appuifw.note(unicode(text))

def liesNeueFrageGUI(alternative):
    """
    Diese Funktion bekommt die zuletzt mgliche Tierart
    als Parameter bergeben und liest von der Nutzerin bzw.
    dem Nutzer eine Frage ein, um die gespeicherte und neue
    Tierart voneinander zu unterscheiden.
    
    Zurckgegeben wird als Tupel:
     - Die neue Tierart
     - Eine Frage, die fr die neue Tierart mit "Ja"
       beantwortet werden muss.
       
    Es wird die Funktion "liesTextGUI" verwendet.
    """
    label = u"Ich kenne die gesuchte Tierart noch nicht. Wie heit sie?"
    neuestier = liesTextGUI(label)
    if neuestier is None:
        return None, None
    label = u"Ja/Nein-Frage mit Antw. 'Ja' fr '"+neuestier+u"' und 'nein' fr '"+alternative+"'?"
    neuefrage = liesTextGUI(unicode(label))
    return neuestier,neuefrage


# Innerer Knoten des Suchbaums. Als Bltter werden Zeichenketten verwendet,
# die nicht als Klasse zusammengefasst werden mssen.
class Frage:
    def __init__(self,text,verweisJA,verweisNEIN):
        self.text = text
        self.verweisJA = verweisJA
        self.verweisNEIN = verweisNEIN


# Im- und Exportfunktionen fr XML-Dateien nach der DTD
# http://www.familie-heming.de/animaltree.dtd
def _animaltreeImportHilfe(element):
    if element.tag == "question":
        if len(element.attrib)!=1:
            raise SyntaxError("'question'-Element muss genau ein Attribut mit Bezeichnung 'txt'"
                              " besitzen, andere Attribute sind nicht erlaubt.")
        if 'txt' not in element.attrib:
            raise SyntaxError("'question'-Element muss genau ein Attribut mit Bezeichnung 'txt'"
                              " besitzen, andere Attribute sind nicht erlaubt.")
        if len(element)!=2:
            raise SyntaxError("'question'-Element muss genau ein 'answerYes'- und 'answerNo'-Tag"
                        " in dieser Reihenfolge enthalten.")
        elif element[0].tag!='answerYes' or element[1].tag!='answerNo':
            raise SyntaxError("'question'-Element muss genau ein 'answerYes'- und 'answerNo'-Tag"
                        " in dieser Reihenfolge enthalten.")
        else:
            retval = Frage(element.attrib['txt'],"dummy","dummy")
            retval.verweisJA = _animaltreeImportHilfe(element[0])
            retval.verweisNEIN = _animaltreeImportHilfe(element[1])
            return retval
    elif element.tag == "animal":
        return element.text
    elif element.tag == "answerYes" or element.tag == "answerNo":
        if len(element.attrib)!=0:
            raise SyntaxError("'"+element.tag+"'-Tags drfen keine Attribute besitzen.")
        elif len(element)!=1:
            raise SyntaxError("'"+element.tag+"'-Tags drfen nur genau ein 'question'- oder"
                              "'animal'-Tag enthalten.")
        elif element[0].tag!='animal' and element[0].tag!='question':
            raise SyntaxError("'"+element.tag+"'-Tags drfen nur genau ein 'question'- oder"
                              "'animal'-Tag enthalten.")
        else:
            return _animaltreeImportHilfe(element[0])
    else:
        raise SyntaxError("Die Tag-Bezeichnung "+element.tag+" ist unbekannt.")

def animaltreeImport(dateiname):
    tree = ET.parse(dateiname)
    root = tree.getroot()
    if root.tag != "animaltree":
        raise SyntaxError("Wurzelelement muss 'animaltree' sein.")
    elif len(root)!= 1:
        raise SyntaxError("Wurzelelement darf nur ein einzelnes 'question'-Tag enthalten.")
    elif root[0].tag!="question":
        raise SyntaxError("Wurzelelement darf nur ein einzelnes 'question'-Tag enthalten.")
    elif len(root.attrib)!=0:
        raise SyntaxError("Wurzelelement darf keine Attribute besitzen.")
    wurzel = _animaltreeImportHilfe(root[0])
    return wurzel

def _animaltreeExportHilfe(knoten):
    if type(knoten)==str or type(knoten)==unicode:
        retval = ET.Element("animal")
        retval.text = knoten
        return retval
    else:
        retval = ET.Element("question")
        retval.attrib['txt'] = knoten.text
        answerYes = ET.Element("answerYes")
        answerYes.append(_animaltreeExportHilfe(knoten.verweisJA))
        answerNo = ET.Element("answerNo")
        answerNo.append(_animaltreeExportHilfe(knoten.verweisNEIN))
        retval.append(answerYes)
        retval.append(answerNo)
        return retval

def animaltreeExport(wurzel,dateiname):
    header = '<?xml version="1.0" encoding="utf-8"?>\n<!DOCTYPE animaltree SYSTEM "http://www.familie-heming.de/animaltree.dtd">\n'
    root = ET.Element("animaltree")
    root.append(_animaltreeExportHilfe(wurzel))
    fh = open(dateiname,"w")
    fh.write(header)
    tree = ET.ElementTree(root)
    tree.write(fh)    


def datenLaden():
    nochmal = True
    while nochmal:
        dateiname = liesTextGUI("Pfadname eingeben:")
        try:
            if dateiname is not None:
                wurzel = animaltreeImport(dateiname)
                gibAusGUI("Daten wurden erfolgreich geladen.")
                return wurzel
            else:
                gibAusGUI("Abbruch: Standarddaten werden verwendet.")
            nochmal = False
        except IOError:
            gibAusGUI("Daten konnten nicht gelesen werden, evtl. einen anderen Pfadnamen verwenden.")
        except SyntaxError, serror:
            gibAusGUI("Daten konnten nicht gelesen werden, Format fehlerhaft.")
            gibAusGUI(serror.message)
        return None # der Vollstaendigkeit halber

def datenSpeichern(wurzel):
    nochmal = True
    while nochmal:
        dateiname = liesTextGUI("Pfadname eingeben (evtl. vorhande Datei wird ueberschrieben!):")
        try:
            if dateiname is not None:
                animaltreeExport(wurzel,dateiname)
                gibAusGUI("Daten wurden erfolgreich gespeichert.")
            nochmal = False
        except IOError:
            gibAusGUI("Daten konnten nicht gespeichert werden, evtl. einen anderen Pfadnamen verwenden.")


# if __name__ == "__main__":
if True:
    # Initialisierung des Wurzelbaums
    # mit mindestens einer Frage
    if stelleFrageGUI("Daten aus Datei laden?"):
        wurzel = datenLaden()
        if wurzel is None:
            wurzel = Frage("Schwimmt es im Wasser?","Fisch","Hund")
    else:
        wurzel = Frage("Schwimmt es im Wasser?","Fisch","Hund")
    nochmal = True
    while nochmal:
        aktKnoten = wurzel
        while type(aktKnoten)!=str and type(aktKnoten)!=unicode:
            # Innerer Knoten, Fragestellen und weiterlaufen
            vorfahre = aktKnoten
            if stelleFrageGUI(aktKnoten.text):
                aktKnoten = aktKnoten.verweisJA
                letzteEntscheidung = True
            else:
                aktKnoten = aktKnoten.verweisNEIN
                letzteEntscheidung = False
        # Blatt erreicht, Tierart gefunden?
        if stelleFrageGUI("Lautet die gesuchte Tierart "+aktKnoten+ "?"):
            gibAusGUI("Das hat ja gut geklappt.")
        else:
            neuestier,neuefrage = liesNeueFrageGUI(aktKnoten)
            if neuestier is None or neuefrage is None:
                gibAusGUI("Hinzufuegen einer Tierart abgebrochen.")
            else:
                neuerKnoten = Frage(neuefrage,neuestier,aktKnoten)
                if letzteEntscheidung:
                    vorfahre.verweisJA = neuerKnoten
                else:
                    vorfahre.verweisNEIN = neuerKnoten
        nochmal = stelleFrageGUI("Spiel von vorne beginnen?")
    if stelleFrageGUI("Evtl. veraenderte Daten speichern?"):
        datenSpeichern(wurzel)

            
            
            
            
            
            
            
            
            